SystemOrganization addCategory: #'FuelDebug-Core'!
SystemOrganization addCategory: #FuelDebug!
SystemOrganization addCategory: #'FuelDebug-Tests'!
SystemOrganization addCategory: #'FuelDebug-Core-Tests'!

!FLEncoder methodsFor: '*FuelDebug' stamp: 'MartinDias 5/11/2012 20:39'!
position
	^ stream position! !

!FLSerializer methodsFor: '*FuelDebug-configuring' stamp: 'MartinDias 5/18/2012 00:42'!
setDebug

	^ serializationFactory := [:anObject :anEncoder | 
		(FLDebugSerialization 
			with: anEncoder
			root: anObject 
			analyzer: self analyzer)
			run;
			yourself ]! !

FLAnalysis subclass: #FLDebugAnalysis
	instanceVariableNames: 'currentlyTracing references'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelDebug-Core'!

!FLDebugAnalysis commentStamp: '<historical>' prior: 0!
I am an analysis which facilitates debugging. 

To use, send #setDebug to an analyzer. Example:

(FLAnalyzer newDefault 
	setDebug;
	analysisFor: #(5 #(7 #(11))))
	pathsTo: 11.

will answer paths to 11.

It's very recommended to use FuelPreview package for visualizing such paths.!

!FLDebugAnalysis methodsFor: 'initialize-release' stamp: 'MartinDias 5/16/2012 01:22'!
initialize
	super initialize.
	references := IdentitySet new.
	Smalltalk globals at: #FLDEBUG_AN put: self.
! !

!FLDebugAnalysis methodsFor: 'mapping' stamp: 'MartinDias 5/16/2012 00:56'!
mapAndTrace: anObject

	currentlyTracing := anObject.
	super mapAndTrace: anObject.
	
	! !

!FLDebugAnalysis methodsFor: 'debugging' stamp: 'MartinDias 5/16/2012 01:33'!
pathsTo: anObject

	^self pathsTo: anObject excluding: #()! !

!FLDebugAnalysis methodsFor: 'debugging' stamp: 'MartinDias 5/16/2012 01:33'!
pathsTo: anObject excluding: excludedObjects

	| referencesToAnObject result |
	(excludedObjects includes: anObject)
		ifTrue: [^Array new].
	referencesToAnObject := references select: [:association |
		 anObject == association value ].

	result := referencesToAnObject.
	referencesToAnObject do: [:association |
		association key == root
			ifFalse: [result addAll: (
				self pathsTo: association key 
				excluding: excludedObjects, {anObject})]].
	^result! !

!FLDebugAnalysis methodsFor: 'debugging' stamp: 'MartinDias 5/16/2012 01:33'!
pathsToEvery: anUnaryBlock

	| referencesToAnObject result |
	referencesToAnObject := references select: [:association | 
		anUnaryBlock value: association value].

	result := referencesToAnObject.
	referencesToAnObject do: [:association |
		association key == root
			ifFalse: [result addAll: (self pathsTo: association key)]].
	^result! !

!FLDebugAnalysis methodsFor: 'debugging' stamp: 'MartinDias 5/16/2012 01:34'!
references
	^ references! !

!FLDebugAnalysis methodsFor: 'accessing' stamp: 'MartinDias 5/18/2012 20:21'!
root
	^ root! !

!FLDebugAnalysis methodsFor: 'tracing' stamp: 'MartinDias 5/16/2012 00:57'!
trace: anObject

	references add: (Association key: currentlyTracing value: anObject).
	super trace: anObject.
	
	! !

!SequenceableCollection methodsFor: '*FuelDebug' stamp: 'MartinDias 5/17/2012 19:35'!
inspectAt: aSmallInteger 
	(self at: aSmallInteger) inspect! !

!FLAnalyzer methodsFor: '*FuelDebug-configuring' stamp: 'MarianoMartinezPeck 9/18/2012 11:01'!
setDebug

	analysisFactory := [:anObject | 
		(FLDebugAnalysis 
			newWith: self firstInMapperChain 
			private: self privateFirstInMapperChain
			root: anObject)
			run;
			yourself ]! !

TestCase subclass: #FLDebugAnalysisTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelDebug-Tests'!

!FLDebugAnalysisTest methodsFor: 'accessing' stamp: 'MartinDias 5/16/2012 01:10'!
analysisOf: anObject

	^ FLAnalyzer newDefault 
		setDebug; 
		analysisFor: anObject
	! !

!FLDebugAnalysisTest methodsFor: 'tests' stamp: 'MartinDias 5/16/2012 01:34'!
testAllReferencesInNestedArray
	
	| array references expectedReferences |
	array := #(1 #(42)).
	references := (self analysisOf: array) references.
	expectedReferences := Set 
		with: #(1 #(42)) -> Array  "todo: this cluster-level references should be removed"
		with: #(1 #(42)) -> 1 
		with: #(1 #(42)) -> #(42) 
		with: #(42) -> 42.
	
	self assert: references = expectedReferences.
! !

!FLDebugAnalysisTest methodsFor: 'tests' stamp: 'MartinDias 5/16/2012 01:32'!
testOnePathInNestedArray
	
	| array references expectedReferences |
	array := #(1 #(42)).
	references := (self analysisOf: array) pathsTo: 42.
	expectedReferences := Set 
		with: #(1 #(42)) -> #(42) 
		with: #(42) -> 42.
	
	self assert: references = expectedReferences.
! !

!FLDebugAnalysisTest methodsFor: 'tests' stamp: 'MartinDias 5/16/2012 01:32'!
testOnePathInNestedArrayWithRepetition
	
	| array references expectedReferences subArray |
	subArray := Array with: 42.
	array := Array with: subArray with: subArray.
	references := (self analysisOf: array) pathsTo: 42.
	expectedReferences := Set 
		with: array -> subArray 
		with: subArray -> 42.
	
	expectedReferences do: [:expectedAssociation |
		self assert: (references anySatisfy: [:association | 
			association key == expectedAssociation key and: [association value == expectedAssociation value]])].! !

!FLDebugAnalysisTest methodsFor: 'tests' stamp: 'MartinDias 5/16/2012 01:33'!
testOneSelectedPath
	
	| array references expectedReferences |
	array := #(42 nil).
	references := (self analysisOf: array) pathsToEvery: [:o | o isNil].
	expectedReferences := Set with: array -> nil.
	
	self assert: references = expectedReferences.
! !

!FLDebugAnalysisTest methodsFor: 'tests' stamp: 'MartinDias 5/16/2012 01:32'!
testTwoPathsInNestedArray
	
	| array references expectedReferences |
	array := #(42 #(42)).
	references := (self analysisOf: array) pathsTo: 42.
	expectedReferences := Set 
		with: #(42 #(42)) -> 42 
		with: #(42 #(42)) -> #(42) 
		with: #(42) -> 42.
	
	self assert: references = expectedReferences.
! !

!FLDebugAnalysisTest methodsFor: 'tests' stamp: 'MartinDias 5/16/2012 01:33'!
testTwoSelectedPaths
	
	| array references expectedReferences |
	array := #(42 nil 1).
	references := (self analysisOf: array) pathsToEvery: [:o | o isNumber].
	expectedReferences := Set with: array -> 42 with: array -> 1.
	
	self assert: references = expectedReferences.
! !

FLMaterialization subclass: #FLDebugMaterialization
	instanceVariableNames: 'log'
	classVariableNames: 'Last'
	poolDictionaries: ''
	category: 'FuelDebug-Core'!

!FLDebugMaterialization commentStamp: '<historical>' prior: 0!
I am a materialization which facilitates debugging, by logging the stream position before and after main steps of FLMaterialization, including cluster information. Obviously, you should be familiar with such class and the algorithm to understand the output log.

To use, send #setDebug to your serializer and run as usually. For example:

FileStream oldFileNamed: 'debug.fuel' do: [:aFile |
	FLMaterializer newDefault
		setDebug;
		materializeFrom: aFile binary ]

Then, inspect the output log:

FLDebugMaterialization last log!

!FLDebugMaterialization classSide methodsFor: 'instance creation' stamp: 'MartinDias 5/17/2012 22:59'!
basicNew 
	^ Last := super basicNew.
	! !

!FLDebugMaterialization classSide methodsFor: 'accessing' stamp: 'MartinDias 5/22/2012 17:18'!
last
	^Last ifNil: [ self error: 'Log not initialized. Read my class comments: ', self class comment ]! !

!FLDebugMaterialization methodsFor: 'debugging' stamp: 'MartinDias 5/17/2012 23:37'!
clusterInstancesStep
	^ self logAndValue: [
		super clusterInstancesStep.
		self log: 'cluster' -> clusters last ].! !

!FLDebugMaterialization methodsFor: 'debugging' stamp: 'MartinDias 5/17/2012 22:54'!
headerStep
	^ self logAndValue: [super headerStep].
! !

!FLDebugMaterialization methodsFor: 'debugging' stamp: 'MartinDias 5/17/2012 23:02'!
initialize
	super initialize.
	log := OrderedCollection new.! !

!FLDebugMaterialization methodsFor: 'debugging' stamp: 'MartinDias 5/18/2012 02:23'!
log
	^ log! !

!FLDebugMaterialization methodsFor: 'debugging' stamp: 'MartinDias 5/12/2012 01:02'!
log: anObject
	log add: anObject! !

!FLDebugMaterialization methodsFor: 'debugging' stamp: 'MartinDias 5/17/2012 23:42'!
logAndValue: aBlock
	| result |
	self log: decoder position -> #before -> aBlock method selector.
	result := aBlock value.
	self log: decoder position -> #after -> aBlock method selector.
	^ result
! !

!FLDebugMaterialization methodsFor: 'debugging' stamp: 'MartinDias 5/17/2012 22:53'!
referencesStep
	^ self logAndValue: [super referencesStep].! !

!FLDebugMaterialization methodsFor: 'debugging' stamp: 'MartinDias 5/17/2012 22:53'!
trailerStep
	^ self logAndValue: [super trailerStep].! !

!FLMaterializer methodsFor: '*FuelDebug-configuring' stamp: 'MartinDias 5/16/2012 01:14'!
setDebug

	materializationFactory := [:aDecoder | 
		(FLDebugMaterialization 
			with: aDecoder)
			run; 
			yourself ]
	
	
! !

!FLDecoder methodsFor: '*FuelDebug' stamp: 'MartinDias 5/12/2012 01:02'!
position
	^ stream position! !

FLSerialization subclass: #FLDebugSerialization
	instanceVariableNames: 'log'
	classVariableNames: 'Last'
	poolDictionaries: ''
	category: 'FuelDebug-Core'!

!FLDebugSerialization commentStamp: '<historical>' prior: 0!
I am a serialization which facilitates debugging, by logging the stream position before and after main steps of FLSerialization, including cluster information. Obviously, you should be familiar with such class and the algorithm to understand the output log.

To use, send #setDebug to your serializer and run as usually. For example:

FileStream forceNewFileNamed: 'debug.fuel' do: [:aFile |
	FLSerializer newDefault
		setDebug;
		serialize: 'hello' on: aFile binary ]

Then, inspect the output log:

FLDebugSerialization last log!

!FLDebugSerialization classSide methodsFor: 'instance creation' stamp: 'MartinDias 5/17/2012 23:00'!
basicNew 
	^ Last := super basicNew.
	! !

!FLDebugSerialization classSide methodsFor: 'accessing' stamp: 'MartinDias 5/22/2012 17:04'!
last
	^Last ifNil: [ self error: 'Log not initialized. Read my class comments: ', self class comment ]! !

!FLDebugSerialization methodsFor: 'debugging' stamp: 'MartinDias 5/17/2012 23:37'!
clusterInstancesStepOf: aCluster
	^ self logAndValue: [
		super clusterInstancesStepOf: aCluster.
		self log: 'cluster' -> aCluster ].
! !

!FLDebugSerialization methodsFor: 'debugging' stamp: 'MartinDias 5/17/2012 22:33'!
headerStep
	^ self logAndValue: [super headerStep]
! !

!FLDebugSerialization methodsFor: 'debugging' stamp: 'MartinDias 5/17/2012 22:55'!
initialize
	super initialize.
	log := OrderedCollection new.! !

!FLDebugSerialization methodsFor: 'debugging' stamp: 'MartinDias 5/18/2012 02:23'!
log
	^ log! !

!FLDebugSerialization methodsFor: 'debugging' stamp: 'MartinDias 5/11/2012 21:20'!
log: anObject
	log add: anObject! !

!FLDebugSerialization methodsFor: 'debugging' stamp: 'MartinDias 5/17/2012 23:43'!
logAndValue: aBlock
	| result |
	self log: encoder position -> #before -> aBlock method selector.
	result := aBlock value.
	self log: encoder position -> #after -> aBlock method selector.
	^ result
! !

!FLDebugSerialization methodsFor: 'debugging' stamp: 'MartinDias 5/17/2012 22:33'!
referencesStep
	^ self logAndValue: [super referencesStep]! !

!FLDebugSerialization methodsFor: 'debugging' stamp: 'MartinDias 5/17/2012 22:33'!
trailerStep
	^ self logAndValue: [super trailerStep]! !
